/*
 * Author: Neil Kolban
 * Date: 6/27/2002
 * 
 * SimpleRequest
 * -------------
 * This application demonstrates sending a request message to a named queue
 * and awaiting a response.  For the response to arrive, a partner application
 * must receive the request and generate the response.  The SimpleResponse
 * application can serve this purpose.
 * 
 * If no response arrives within 15 seconds, the request is assumed to have
 * timed out.
 * 
 * Usage: SimpleRequest RequestQName ResponseQName [QmgrName]
 * 
 */
using System;
using MQ;

namespace MQSamples
{
	/// <summary>
	/// Put a request message to a queue and wait for a response.
	/// </summary>
	class SimpleRequest
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			if (args.Length < 2)
			{
				Console.WriteLine("Usage: SimpleRequest RequestQName ResponseQName [QmgrName]");
				return;
			}
			string requestQName = args[0];
			string responseQName = args[1];
			string qMgrName = "";
			if (args.Length > 2)
			{
				qMgrName = args[2];
			}

			try
			{
				Console.WriteLine("Sample SimpleRequest started");

				MQQueueManager qMgr = new MQQueueManager(qMgrName);
				MQQueue requestQueue = qMgr.AccessQueue(requestQName, MQC.MQOO_OUTPUT);
				MQQueue responseQueue = qMgr.AccessQueue(responseQName, MQC.MQOO_INPUT_AS_Q_DEF);



				Console.WriteLine("Enter request text: ");
				string line = Console.ReadLine();

				MQMessage requestMessage = new MQMessage();
				requestMessage.WriteString(line);
				requestMessage.MessageType = MQC.MQMT_REQUEST;
				requestMessage.Report = MQC.MQRO_COPY_MSG_ID_TO_CORREL_ID;
				requestMessage.ReplyToQName = responseQName;
				requestQueue.Put(requestMessage);
				requestQueue.Close();

				Console.WriteLine("Request sent, waiting for response ...");

				MQMessage responseMessage = new MQMessage();
				responseMessage.CorrelId = requestMessage.MsgId;

				MQGetMessageOptions gmo = new MQGetMessageOptions();
				gmo.Options = MQC.MQGMO_WAIT;
				gmo.WaitInterval = 15 * 1000;
				gmo.MatchOptions = MQC.MQMO_MATCH_CORREL_ID;

				try
				{
					responseQueue.Get(responseMessage, gmo);
					line = responseMessage.ReadString(responseMessage.MessageLength);
					Console.WriteLine("Response: {0}", line);
				}
				catch (MQException e)
				{
					if (e.Reason == MQException.MQRC_NO_MSG_AVAILABLE)
					{
						Console.WriteLine("No response received");
					}
					else
					{
						throw e;
					}
				}
				responseQueue.Close();
				qMgr.Disconnect();
			}
			catch(MQException e)
			{
				Console.WriteLine("Error: {0}", e.ToString());
			}
		}
	}
}
